@@ -1,11 +1,11 @@
-__version__ = '0.9.1'
+__version__ = "0.19.2"
+
 from sparkmagic.serverextension.handlers import load_jupyter_server_extension
 
 
 def _jupyter_server_extension_paths():
-    return [{
-        "module": "sparkmagic"
-    }]
+    return [{"module": "sparkmagic"}]
+
 
 def _jupyter_nbextension_paths():
     return []
