@@ -1,9 +1,6 @@
-import json
-import pandas as pd
-from collections import OrderedDict
 from hdijupyterutils.guid import ObjectWithGuid
 
-from sparkmagic.utils.utils import coerce_pandas_df_to_numeric_datetime
+from sparkmagic.utils.utils import coerce_pandas_df_to_numeric_datetime, records_to_dataframe
 import sparkmagic.utils.configuration as conf
 import sparkmagic.utils.constants as constants
 from sparkmagic.utils.sparkevents import SparkEvents
@@ -12,7 +9,7 @@ from .exceptions import DataFrameParseException, BadUserDataException
 
 
 class SQLQuery(ObjectWithGuid):
-    def __init__(self, query, samplemethod=None, maxrows=None, samplefraction=None, spark_events=None):
+    def __init__(self, query, samplemethod=None, maxrows=None, samplefraction=None, spark_events=None, coerce=None):
         super(SQLQuery, self).__init__()
         
         if samplemethod is None:
@@ -36,16 +33,15 @@ class SQLQuery(ObjectWithGuid):
         if spark_events is None:
             spark_events = SparkEvents()
         self._spark_events = spark_events
+        self._coerce = coerce
 
     def to_command(self, kind, sql_context_variable_name):
         if kind == constants.SESSION_KIND_PYSPARK:
             return self._pyspark_command(sql_context_variable_name)
-        elif kind == constants.SESSION_KIND_PYSPARK3:
-            return self._pyspark_command(sql_context_variable_name, False)
         elif kind == constants.SESSION_KIND_SPARK:
             return self._scala_command(sql_context_variable_name)
         elif kind == constants.SESSION_KIND_SPARKR:
-            return self._r_command()
+            return self._r_command(sql_context_variable_name)
         else:
             raise BadUserDataException(u"Kind '{}' is not supported.".format(kind))
 
@@ -56,10 +52,10 @@ class SQLQuery(ObjectWithGuid):
         try:
             command = self.to_command(session.kind, session.sql_context_variable_name)
             command_guid = command.guid
-            (success, records_text) = command.execute(session)
+            (success, records_text, mimetype) = command.execute(session)
             if not success:
                 raise BadUserDataException(records_text)
-            result = self._records_to_dataframe(records_text, session.kind)
+            result = records_to_dataframe(records_text, session.kind, self._coerce)
         except Exception as e:
             self._spark_events.emit_sql_execution_end_event(session.guid, session.kind, session.id, self.guid,
                                                             command_guid, False, e.__class__.__name__, str(e))
@@ -70,45 +66,23 @@ class SQLQuery(ObjectWithGuid):
                                                             command_guid, True, "", "")
             return result
 
-    @staticmethod
-    def _records_to_dataframe(records_text, kind):
-        if records_text == '':
-            strings = []
-        else:
-            strings = records_text.split('\n')
-        try:
-            data_array = [json.JSONDecoder(object_pairs_hook=OrderedDict).decode(s) for s in strings]
 
-            if kind == constants.SESSION_KIND_SPARKR and len(data_array) > 0:
-                data_array = data_array[0]
-
-            if len(data_array) > 0:
-                df = pd.DataFrame(data_array, columns=data_array[0].keys())
-            else:
-                df = pd.DataFrame(data_array)
-            
-            coerce_pandas_df_to_numeric_datetime(df)
-            return df
-        except ValueError:
-            raise DataFrameParseException(u"Cannot parse object as JSON: '{}'".format(strings))
-
-    def _pyspark_command(self, sql_context_variable_name, encode_result=True):
-        command = u'{}.sql(u"""{} """).toJSON()'.format(sql_context_variable_name, self.query)
+    def _pyspark_command(self, sql_context_variable_name):
+        # use_unicode=False means the result will be UTF-8-encoded bytes, so we
+        # set it to False for Python 2.
+        command = u'{}.sql(u"""{} """).toJSON(use_unicode=(sys.version_info.major > 2))'.format(
+            sql_context_variable_name, self.query)
         if self.samplemethod == u'sample':
             command = u'{}.sample(False, {})'.format(command, self.samplefraction)
         if self.maxrows >= 0:
             command = u'{}.take({})'.format(command, self.maxrows)
         else:
             command = u'{}.collect()'.format(command)
-        # Unicode support has improved in Python 3 so we don't need to encode.
-        if encode_result:
-            print_command = '{}.encode("{}")'.format(constants.LONG_RANDOM_VARIABLE_NAME,
-                                                     conf.pyspark_sql_encoding())
-        else:
-            print_command = constants.LONG_RANDOM_VARIABLE_NAME
-        command = u'for {} in {}: print({})'.format(constants.LONG_RANDOM_VARIABLE_NAME,
-                                                    command,
-                                                    print_command)
+        print_command = constants.LONG_RANDOM_VARIABLE_NAME
+        command = u'import sys\nfor {} in {}: print({})'.format(
+            constants.LONG_RANDOM_VARIABLE_NAME,
+            command,
+            print_command)
         return Command(command)
 
     def _scala_command(self, sql_context_variable_name):
@@ -121,8 +95,11 @@ class SQLQuery(ObjectWithGuid):
             command = u'{}.collect'.format(command)
         return Command(u'{}.foreach(println)'.format(command))
 
-    def _r_command(self):
-        command = u'sql("{}")'.format(self.query)
+    def _r_command(self, sql_context_variable_name):
+        if sql_context_variable_name == 'spark':
+            command = u'sql("{}")'.format(self.query)
+        else:
+            command = u'sql({}, "{}")'.format(sql_context_variable_name, self.query)
         if self.samplemethod == u'sample':
             command = u'sample({}, FALSE, {})'.format(command, self.samplefraction)
         if self.maxrows >= 0:
@@ -138,7 +115,8 @@ class SQLQuery(ObjectWithGuid):
         return self.query == other.query and \
             self.samplemethod == other.samplemethod and \
             self.maxrows == other.maxrows and \
-            self.samplefraction == other.samplefraction
+            self.samplefraction == other.samplefraction and \
+            self._coerce == other._coerce
 
     def __ne__(self, other):
         return not (self == other)
